# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.forms import HiddenInput, MultipleHiddenInput, Widget
from django.forms.utils import flatatt
from django.utils.encoding import force_text
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _


class FilterWidgetMixin(object):
    class Media:
        js = ("js/foreign_key_widget.js",)

    @staticmethod
    def render_input_filter():
        return format_html('<input type="text" class="input-filter" '
                           'style="width:286px;display:block" placeholder="{}" />'
                           .format(_("Search")))

    @staticmethod
    def render_input_filter_results():
        return format_html(
            '<select multiple class="filter-result" style="width:300px;display:block"></select>')

    def render_selected_options(self, values):
        if values is None or values == "":
            return ""
        if not isinstance(values, list):
            values = [values]
        items_selected = []
        choices = self.choices.queryset.filter(pk__in=values)
        for choice in choices:
            option_value, option_label = self.choices.choice(choice)
            items_selected.append('<li value="{}">{}</li>'.format(option_value, force_text(option_label)))
        return '\n'.join(items_selected)

    def render_selected_widget(self, values):
        items_selected = self.render_selected_options(values)
        return format_html('<ul style="display:block">{}</ul>'.format(items_selected))

    def render_filter_widget(self, values, widget, name, attrs=None):
        final_attrs = self.build_attrs(attrs, name=name, multiple=self.is_multiple)
        selected_items = self.render_selected_widget(values)
        input_filter = self.render_input_filter()
        input_filter_results = self.render_input_filter_results()
        return format_html('<div class="filter-widget" {}>'
                           '<div class="form-send">{}</div> '
                           '{selected_items} {input_filter} '
                           '<div class="loading" style="visibility:hidden">{loading}...</div> '
                           '{input_filter_results}'
                           '</div>'
                           .format(flatatt(final_attrs),
                                   widget,
                                   selected_items=selected_items,
                                   input_filter=input_filter,
                                   loading=_('Loading'),
                                   input_filter_results=input_filter_results))


class ForeignKeyWidget(HiddenInput, FilterWidgetMixin):
    is_multiple = 0

    @property
    def is_hidden(self):
        return False

    def render(self, name, value, attrs=None):
        hidden_input = super(ForeignKeyWidget, self).render(name, value, attrs)
        return self.render_filter_widget(values=value, widget=hidden_input, name=name, attrs=attrs)


class ManyToManyWidget(MultipleHiddenInput, FilterWidgetMixin):
    is_multiple = 1

    @property
    def is_hidden(self):
        return False

    def render(self, name, value, attrs=None):
        hidden_inputs = super(ManyToManyWidget, self).render(name, value, attrs)
        return self.render_filter_widget(values=value, widget=hidden_inputs, name=name, attrs=attrs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import

import random
import string


def random_string(length=16):
    """ Generates a random alphanumeric string.

    :param length: Length of the generated string.
    """
    chars = string.digits + string.ascii_letters
    return ''.join(random.choice(chars) for _ in range(length))


def random_pin(length=4):
    """ Generates a random numeric string

    :param length: Length of the generated string.
    """
    chars = string.digits
    return ''.join(random.choice(chars) for _ in range(length))


def unique_random_string(model, field, length=16):
    """ Generates a random alphanumeric string, unique in code field from a django class model

    :param length: Length of the generated string.
    :param model: Model class, it has a code field.
    :param field: field name to be sure is unique.
    """
    value = random_string(length)
    while model.objects.filter(**{field: value}).exists():
        length += random.choice([-1, 1])
        value = random_string(length)
    return value



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import

import os
import subprocess
from distutils.log import INFO

import setuptools
from setuptools import setup
from setuptools.command.build_py import build_py
from setuptools.command.sdist import sdist

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

with open(os.path.join(os.path.dirname(__file__), 'HISTORY.rst')) as history:
    HISTORY = history.read().replace('.. :changelog:', '')

# Allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

# Dynamically calculate the version based on belt.VERSION.
version = __import__('belt').__version__


class InstallStaticDependenciesCommand(setuptools.Command):
    """A custom command to run npm build command for static files."""

    description = 'run npm run build to compile static files of the package'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        """Run command."""
        command = ['npm', 'install']
        self.announce(
            'Running command: %s' % str(command),
            level=INFO)
        subprocess.check_call(command)


class BuildStaticCommand(setuptools.Command):
    """A custom command to run npm build command for static files."""

    description = 'run npm run build to compile static files of the package'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        """Run command."""
        command = ['npm', 'run', 'build']
        self.announce(
            'Running command: %s' % str(command),
            level=INFO)
        subprocess.check_call(command)


class BuildPyCommand(build_py):
    """Custom build command."""

    def run(self):
        self.run_command('install_static')
        self.run_command('build_static')
        setuptools.command.build_py.build_py.run(self)


class SDistCommand(sdist):
    """Custom sdist command."""

    def run(self):
        self.run_command('install_static')
        self.run_command('build_static')
        setuptools.command.sdist.sdist.run(self)


setup(
    name='django-belt',
    version=version,
    packages=[
        'belt',
    ],
    include_package_data=True,
    license='MIT License',
    description='Simple package with some utilities for Django',
    long_description=README + '\n\n' + HISTORY,
    url='https://github.com/marcosgabarda/django-belt',
    author='Marcos Gabarda',
    author_email='hey@marcosgabarda.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Utilities',
    ],
    install_requires=[
        'django>=1.9',
        'django-model-utils>=2.0',
    ],
    cmdclass={
        'install_static': InstallStaticDependenciesCommand,
        'build_static': BuildStaticCommand,
        'build_py': BuildPyCommand,
        'sdist': SDistCommand,
    }
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import

import os
import shutil


def delete_after(filename):
    """Decorator to be sure the file given by parameter is deleted after the
    execution of the method.
    """
    def delete_after_decorator(function):
        def wrapper(*args, **kwargs):
            try:
                return function(*args, **kwargs)
            finally:
                if os.path.isfile(filename):
                    os.remove(filename)
                if os.path.isdir(filename):
                    shutil.rmtree(filename)
        return wrapper
    return delete_after_decorator
